'''
Christian Pluchar- LIGO SURF17


Image Analysis for Camera Calibration
Currently takes a group of images spread among different directories, adds up the RAW file values
at each pixel location, then averages out the counts between images of equal exposure
to give an indication of the amount of light hitting the sensor. Adding functionality
to get data from each of the subpixels (red, green, blue). Also performs a 
rudimentary check to make sure the pixels are not overexposed (checks them against
the saturated value of 2^14-1 because of the nikon camera used has 14 bits assigned 
per pixel when shooting in the RAW mode selected). Prints the data at the end of the run,
further changes are expected to export the data into a NPY file. 


Rawpy (for opening Raw images and extracting data from them)
imageoio (Rawpy uses this)
Numpy (for some simple data processing)
Glob (included, for opening groups of images)


'''

import rawpy
import imageio
import numpy as np
from PIL import Image
import glob
import os
import csv

#seperate file containing code to produce the graphs
# from makePlotsD810 import *

RED = 0
GREEN = 1
BLUE = 2

directory = 'data_7_7//all//'
saturated_value = 2**14 - 1
num_channels = 3


#names of folder paths to analyze
folders = [
'nhigh320',
'nhigh640',
'nhigh1250',
'nhigh2500',
'nhigh5000',
'nhigh8000',
'nlow320',
'nlow640',
'nlow1250',
'nlow2500',
'nlow5000',
'nlow8000',
'nmed320',
'nmed640',
'nmed1250',
'nmed2500',
'nmed5000',
'nmed8000',
'whigh320',
'whigh640',
'whigh1250',
'whigh2500',
'whigh5000',
'whigh8000',
'wlow320',
'wlow640',
'wlow1250',
'wlow2500',
'wlow5000',
'wlow8000',
'wmed320',
'wmed640',
'wmed1250',
'wmed2500',
'wmed5000',
'wmed8000'
]

#keeps track of where to add the next image in the 2d array of allImages
counter = 0 

#stores where all the images are
allImages = []



#gets all the images organized in the folders and adds them to allImages
#NOTE: will have to change the directory back the location of analysis.py
for folder in folders:
	os.chdir(directory+folder)
	allImages.append([])
	for image in glob.glob('*.NEF'):
		allImages[counter].append(image)
		print(image)
	os.chdir('..')
	os.chdir('..')
	os.chdir('..')
	counter = counter + 1
	
#enter the power levels (in watts) of each of the folders
powerLevels = np.array([
	24.65*10**-6,24.65*10**-6,24.65*10**-6,24.65*10**-6,24.65*10**-6,24.65*10**-6,
	3.55*10**-6,3.55*10**-6,3.55*10**-6,3.55*10**-6,3.55*10**-6,3.55*10**-6,
	14.3*10**-6,14.3*10**-6,14.3*10**-6,14.3*10**-6,14.3*10**-6,14.3*10**-6,
	41.55*10**-6,41.55*10**-6,41.55*10**-6,41.55*10**-6,41.55*10**-6,41.55*10**-6,
	14.95*10**-6, 14.95*10**-6, 14.95*10**-6, 14.95*10**-6, 14.95*10**-6, 14.95*10**-6, 
	29.35*10**-6, 29.35*10**-6, 29.35*10**-6, 29.35*10**-6, 29.35*10**-6, 29.35*10**-6])

#Enter the exposure times (in seconds) for each of the folders
#1/1250, 1/2000, 1/2500, 1/3000
exposureTimes = np.array([
1/320, 1/640, 1/1250, 1/2500, 1/5000, 1/8000,
1/320, 1/640, 1/1250, 1/2500, 1/5000, 1/8000,
1/320, 1/640, 1/1250, 1/2500, 1/5000, 1/8000,
1/320, 1/640, 1/1250, 1/2500, 1/5000, 1/8000,
1/320, 1/640, 1/1250, 1/2500, 1/5000, 1/8000,
1/320, 1/640, 1/1250, 1/2500, 1/5000, 1/8000,
])

# for index in range(0, len(folders)):
# 	print(folder, 'power:', powerLevels[index], 'exposure:', exposureTimes[index])

#function for writing data to a csv file 
def writeData(directory, name, dataToWrite):
	with open(directory + name + '.csv', 'w') as f:
		writer = csv.writer(f)
		writer.writerows(dataToWrite)

#main method for analyzing the images
#returns a list with: folderName, allMean, redMean, greenMean, blueMean
#for a folder of images
def runAnalysis(folder, folderName):

	print(folder, 'contains ', len(folder), 'images in it')

	#keeps track of all of the pixel values extracted from the image
	#Ends up being a 2d list
	#0 -red
	#1 -green
	#2 -blue
	npPixelCounts = np.zeros((num_channels, len(folder)))
	imageNumber = 0
	for path in folder:
		saturated = None
		imagePath = directory + folderName + '//' + path
		readImage = rawpy.imread(imagePath)

		print('Reading ', imagePath)

		#keeps the total for the raw pixel value extracted from each of the sites

		pixel_sum_blue = 0
		pixel_sum_green = 0
		pixel_sum_red = 0

		#holds the pixel values and is added to the array
		#can streamline this later
		tempList = [0,0,0]

		#!!!!!!taken from cj's code

		# Colour filter array
		cfa = readImage.raw_colors
		cfa_shape = cfa.shape
		all_colors = readImage.raw_image

		# Create boolean masks for each colour
		red_mask = np.reshape(np.in1d(cfa, 0), cfa_shape)
		green_mask = np.reshape(np.in1d(cfa, (1, 3)), cfa_shape)
		blue_mask = np.reshape(np.in1d(cfa, 2), cfa_shape)


		# Multiplying original array by masks sets all pixels 
		#   of colours other than the desired colour to 0
		red_only = np.multiply(all_colors, red_mask)
		green_only = np.multiply(all_colors,  green_mask)
		blue_only = np.multiply(all_colors, blue_mask)

		#end cj's code

		for height in range(0, readImage.sizes[0]):
			for width in range(0, readImage.sizes[1]):
				if readImage.raw_value_visible(height,width) == saturated_value:
					saturated = True

				#analysis - for total pixel sum at each site, use option 1, for 
				#getting each color, use option 2
				#1:
				#pixel_sum += readImage.raw_value_visible(height,width)

				#2:
				pixel_sum_red += red_only[height,width]
				pixel_sum_green += green_only[height,width]
				pixel_sum_blue += blue_only[height,width]


		if saturated:
			print(imagePath, ' has overexposed/saturated pixels')

		print('red', pixel_sum_red)
		print('green', pixel_sum_green)
		print('blue', pixel_sum_blue)

		npPixelCounts[RED, imageNumber] = pixel_sum_red
		npPixelCounts[GREEN, imageNumber] = pixel_sum_green
		npPixelCounts[BLUE, imageNumber] = pixel_sum_blue


		#resets it for the next image

		pixel_sum_blue = 0
		pixel_sum_green = 0
		pixel_sum_red = 0
		imageNumber += 1

	#find the mean and stDev for a group of images across all channels


	redMean = np.mean(npPixelCounts[RED])
	redStDevOfMean = np.std(npPixelCounts[RED], ddof=1) / np.sqrt(len(folder))

	greenMean = np.mean(npPixelCounts[GREEN])
	greenStDevOfMean = np.std(npPixelCounts[GREEN], ddof=1) / np.sqrt(len(folder))

	blueMean = np.mean(npPixelCounts[BLUE])
	blueStDevOfMean = np.std(npPixelCounts[BLUE], ddof=1) / np.sqrt(len(folder))

	print('red stdev', redStDevOfMean)
	print(npPixelCounts[RED])
	print(np.std(npPixelCounts[RED]))

	return redMean, greenMean, blueMean, redStDevOfMean, greenStDevOfMean, blueStDevOfMean

#keeps track of which folder is being worked on
folderNumber = 0

dataTableToSave = []

#same data table, just without the names of the folders so it can be a numpy array
#rows = length of all images (number of folders)
#columns = number of color channeles 
dataTableToAnalyze = np.empty((len(allImages), 2 * num_channels))

#Analyzes each folder and puts the data into a 3d array called dataTable
#column 0 is red, 1 is green, 2 is blue
#rows are the different folders
#row, column
for folderName in allImages:
	redMean, greenMean, blueMean, redStDev, greenStDev, blueStDev = runAnalysis(folderName, folders[folderNumber])
	dataTableToSave.append([])
	dataTableToSave[folderNumber].append([folderName, redMean, greenMean, blueMean, redStDev, greenStDev, blueStDev])

	dataTableToAnalyze[folderNumber, RED] = redMean
	dataTableToAnalyze[folderNumber, GREEN] = greenMean
	dataTableToAnalyze[folderNumber, BLUE] = blueMean

	dataTableToAnalyze[folderNumber, RED + num_channels] = redStDev 
	dataTableToAnalyze[folderNumber, GREEN + num_channels] = greenStDev
	dataTableToAnalyze[folderNumber, BLUE + num_channels] = blueStDev
	folderNumber += 1

print(dataTableToAnalyze)


# #writes data counts data to csv file
# with open(directory + 'data.csv', 'w') as f:
#     writer = csv.writer(f)
#     #writer.writerows('allMean', 'redMean', 'greenMean', 'blueMean')
#     for column in dataTable:
#     	writer.writerows(column)

np.save('D7100PixelCounts', dataTableToAnalyze)

#writeData(directory, 'data', dataTableToSave)

# #usues makePlots.py to make the plots of the data
# poptR, poptG, poptB = makePlotsD810(dataTableToAnalyze, powerLevels, exposureTimes)


# writeData(directory, 'fitParameters', np.hstack([poptR, poptG, poptB]))




