

import matplotlib
import numpy as np
matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from scipy.optimize import curve_fit


#in seconds
exposure_times = np.array([1/25, 1/30, 1/40, 1/60, 1/125, 1/250, 1/500, 1/1000, 1/2000, 1/3200, 
	1/5000, 1/6400, 1/8000, 1/15, 1/5, 1/2, 1, 1/10, 2/5])

powers_old = np.array([6.78805616e-06, 6.56406448e-06, 7.34159164e-06, 7.00020947e-06, 
8.07134027e-06, 9.38118373e-06, 1.04605548e-05, 1.33625163e-05, 
2.56629967e-05, 3.44275590e-05, 3.89775695e-05, 4.76430887e-05, 
6.76289439e-05, 6.57596034e-06, 6.50720176e-06, 5.44631064e-06, 
5.71370802e-06, 6.76287202e-06, 6.14375413e-06])

powers = np.array([1.35135671e-06, 1.30847067e-06, 1.46534968e-06, 1.40361328e-06, 
1.63473331e-06, 1.93069024e-06, 2.21762158e-06, 2.93864169e-06, 
5.66700026e-06, 7.75381975e-06, 9.18187606e-06, 1.13076577e-05, 
1.57334386e-05, 1.30643310e-06, 1.28989027e-06, 1.07895523e-06, 
1.13160705e-06, 1.34197730e-06, 1.21719548e-06])

#9, 10, 11, 12 have large spread

transmitivity_factor = .198
ambient_background_subtraction = True
dark_offset_subtraction = True

#in watts
ambient_background_power = 1.81610035e-08


#rgb
#formula from dark offset measurements with D7100
#See August 10th notes
def darkOffsetFunc(exposureTime):
	return  10**(-1 * np.log10(exposureTime) - 8.83)

if dark_offset_subtraction:

	for index in range(0, powers.size):

		powers[index] -= darkOffsetFunc(exposure_times[index])


for index in range(0, len(powers)):
	if ambient_background_subtraction:
		powers[index] -= ambient_background_power


	powers[index] = powers[index] / transmitivity_factor



#print(powers)

energies = np.multiply(powers, exposure_times)
def line(x, slope, yIntersect):
	return (x * slope + yIntersect)

plot1 = plt.figure(1)

plt.scatter(exposure_times, powers)
plt.title('Scattered Power vs Exposure Time (ETMx)')
plt.xlabel('Exposure Time (S)')
plt.ylabel('Scattered Power (W)')
plt.ylim(0, 1e-05)
plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
#plt.xlim(0, .125)


# popt, pcov = curve_fit(line, exposure_times, powers)

# xfine = np.linspace(0, np.amax(exposure_times), num = 1000)
# plt.plot(xfine, line(xfine, *popt), 'r-')

# perr = np.sqrt(np.diag(pcov))

# print(popt)
# print(perr)

plt.show()